package com.mortals.xhx.module.perform.service.impl;
import com.mortals.xhx.common.code.CheckStatusEnum;
import com.mortals.xhx.common.code.SubMethodEnum;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.check.model.CheckComplainRecordEntity;
import com.mortals.xhx.module.check.service.CheckComplainRecordService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.perform.dao.PerformComplainRecordDao;
import com.mortals.xhx.module.perform.model.PerformComplainRecordEntity;
import com.mortals.xhx.module.perform.service.PerformComplainRecordService;
import lombok.extern.slf4j.Slf4j;

import java.util.Date;

/**
* PerformComplainRecordService
* 评价绩效投诉记录信息 service实现
*
* @author zxfei
* @date 2023-07-11
*/
@Service("performComplainRecordService")
@Slf4j
public class PerformComplainRecordServiceImpl extends AbstractCRUDServiceImpl<PerformComplainRecordDao, PerformComplainRecordEntity, Long> implements PerformComplainRecordService {

    @Autowired
    private CheckComplainRecordService checkComplainRecordService;

    @Override
    protected void saveAfter(PerformComplainRecordEntity entity, Context context) throws AppException {
        CheckComplainRecordEntity checkComplainRecordEntity = new CheckComplainRecordEntity();
        checkComplainRecordEntity.initAttrValue();
        BeanUtils.copyProperties(entity,checkComplainRecordEntity, BeanUtil.getNullPropertyNames(entity));
        checkComplainRecordEntity.setId(null);
        checkComplainRecordEntity.setRecordId(entity.getId());
        if(entity.getSubMethod()== SubMethodEnum.系统自动.getValue()){
            checkComplainRecordEntity.setCheckStatus(CheckStatusEnum.已处理.getValue());     //自动扣分相设置为已处理
        }else {
            checkComplainRecordEntity.setCheckStatus(CheckStatusEnum.未处理.getValue());      //非自动扣分相设置为未处理
        }
        checkComplainRecordService.save(checkComplainRecordEntity,context);
    }

    @Override
    public void updateProcessStatus(Long id, Integer status) throws AppException {
        if(id==null){
            throw new AppException("评价绩效投诉记录Id不能为空");
        }
        PerformComplainRecordEntity entity = this.get(id);
        if(entity==null){
            throw new AppException("无效的评价绩效投诉记录Id");
        }
        if(status==null){
            status=1; //处理状态(1.未核查,2.已核查)
        }
        PerformComplainRecordEntity update = new PerformComplainRecordEntity();
        update.setId(id);
        update.setProcessStatus(status);
        update.setUpdateTime(new Date());
        dao.update(update);
    }

}